#!/bin/sh
#-
# Copyright (c) 2009-2017 Hans Ottevanger
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# Cygwin functionality contributed and tested by Antonio Eleuteri.

# This script builds a Minix-ST 1.6.25 cross development environment, based
# on modern versions of GNU gcc and binutils.
# It downloads the required distribution files if not already present.
# This version is still a bit primitive and only lightly tested, so take care.
#
# !!! DO NOT RUN THIS SCRIPT WITH SUPERUSER PRIVILEGES !!!
#

# For now, exit on every reported failure.
set -e

# PREFIX is the directory in which the tools are finally installed.
# The user can configure it at will, but keep in mind that it is completely
# deleted by this script, so it had better have no pre-existing content of
# any value.

PREFIX=$HOME/crosstools

# You could configure a local GNU mirror here ...
GNUSITE=ftp.gnu.org
#GNUSITE=ftp.nluug.nl

BINUTILS=binutils-2.30
#GCC=gcc-4.8.5
#GCC=gcc-4.9.4
#GCC=gcc-5.5.0
#GCC=gcc-6.4.0
#GCC=gcc-7.3.0
GCC=gcc-8.1.0
MPFR=mpfr-4.0.1
GMP=gmp-6.1.2
MPC=mpc-1.1.0
MINIXST=minix-st-1.6.25.HO.v1.3

TARGET=m68k-atari-minix

LANGUAGES="c,c++,fortran"

SYSROOT=$PREFIX/$TARGET/sys-root
TOPDIR=$(pwd)

case $(uname -s) in
FreeBSD)
#	export CC=gcc6 CXX=g++6 LD_LIBRARY_PATH=/usr/local/lib/gcc6
#	export CC=gcc7 CXX=g++7 LD_LIBRARY_PATH=/usr/local/lib/gcc7
	export CC=cc CXX=c++
	MAKE=gmake
	PAR=${PAR-"-j $(sysctl -n hw.ncpu)"}
	;;
OpenBSD)
	export CC=egcc CXX=eg++
	MAKE=gmake
	PAR=${PAR-"-j $(sysctl -n hw.ncpu)"}
	;;
Linux)
	MAKE=make
	PAR=${PAR-"-j $(nproc)"}
	;;
Darwin)
	MAKE=make
	PAR=${PAR-"-j $(sysctl -n hw.ncpu)"}
	;;
CYGWIN*)
	MAKE=make
	PAR=${PAR-"-j $(nproc)"}
	;;
*)
	echo "Don't know how to set environment for $(uname -s)"
	exit 1
	;;
esac

download()
{
	if [ ! -d $2 ]
	then
		echo "Directory $2 does not exist"
		exit 1
	fi

	if [ -f $2/$(basename $1) ]
	then
		echo "$(basename $1) already present in directory $2"
	else
		case $(uname -s) in
		FreeBSD)
			echo -n "Downloading $1 ..."
			fetch -q -o $2 $1
			echo " done"
			;;
		OpenBSD)
			echo -n "Downloading $1 ..."
			ftp -V -o $2/$(basename $1) $1
			echo " done"
			;;
		Linux)
			echo -n "Downloading $1 ..."
			wget -q -P $2 $1
			echo " done"
			;;
		Darwin)
			echo "Downloading $1 ...\c"
			curl -s -o $2/$(basename $1) $1
			echo " done"
			;;
		CYGWIN*)
			echo -n "Downloading $1 ..."
			wget -q -P $2 $1
			echo " done"	
			;;
		*)
			echo "Don't know how to download for $(uname -s)"
			exit 1
			;;
		esac
	fi
}

applypatches()
{
	case $(uname -s) in
	FreeBSD)
		echo "Apply patches from $1 ..."
		patch -p0 < $1
		;;
	OpenBSD)
		echo "Apply patches from $1 ..."
		patch -p0 < $1
		;;
	Linux)
		echo "Apply patches from $1 ..."
		patch -p0 --verbose < $1
		;;
	Darwin)
		echo "Apply patches from $1 ..."
		patch -p0 --verbose < $1
		;;
	CYGWIN*)
		echo "Apply patches from $1 ..."
		patch -p0 --verbose < $1
		;;	
	*)
		echo "Don't know how to apply patches for $(uname -s)"
		exit 1
		;;
	esac
}

rm -rf $PREFIX

mkdir -p $SYSROOT/usr

if [ ! -d distfiles ]
then
	mkdir distfiles
fi

download ftp://$GNUSITE/pub/gnu/binutils/$BINUTILS.tar.bz2 distfiles
# GCC changed its distribution format from 6.4.0 onwards ...
case $GCC in
gcc-5.5.0|gcc-6.4.0|gcc-7.3.0|gcc-8.1.0)
	download ftp://$GNUSITE/pub/gnu/gcc/$GCC/$GCC.tar.xz distfiles
	;;
*)
	download ftp://$GNUSITE/pub/gnu/gcc/$GCC/$GCC.tar.bz2 distfiles
	;;
esac
download ftp://$GNUSITE/pub/gnu/mpfr/$MPFR.tar.bz2 distfiles 
download ftp://$GNUSITE/pub/gnu/gmp/$GMP.tar.bz2 distfiles
download ftp://$GNUSITE/pub/gnu/mpc/$MPC.tar.gz distfiles

download http://www.beastielabs.net/download/$MINIXST.tar.gz distfiles

rm -rf $BINUTILS
tar xjf distfiles/$BINUTILS.tar.bz2

if [ -f diffs/$BINUTILS.diff ]
then
	applypatches diffs/$BINUTILS.diff
else
	echo "no diffs for $BINUTILS"
	exit 1
fi

rm -rf bobjdir

mkdir bobjdir

cd bobjdir

$TOPDIR/$BINUTILS/configure --prefix=$PREFIX --target=$TARGET \
	--with-sysroot --with-system-zlib \
	--disable-nls \
	--disable-werror # prevent failure when compiling with gcc-4.9.x

$MAKE $PAR
$MAKE install

cd $TOPDIR

rm -rf minix-1.6.25
mkdir -p minix-1.6.25/usr
tar xzf distfiles/$MINIXST.tar.gz -C minix-1.6.25/usr
cp -Rp minix-1.6.25/usr/include $SYSROOT/usr

cd utils
$MAKE
cp cv msize mnm mstrip chmem $PREFIX/bin
cp cv $PREFIX/$TARGET/bin
$MAKE clean
cd $TOPDIR

mkdir -p $SYSROOT/usr/lib
cp files/ldscript $SYSROOT/usr/lib

rm -rf $GCC
# GCC changed its distribution format from 6.4.0 onwards ...
case $GCC in
gcc-5.5.0|gcc-6.4.0|gcc-7.3.0|gcc-8.1.0)
	tar xJf distfiles/$GCC.tar.xz
	;;
*)
	tar xjf distfiles/$GCC.tar.bz2
	;;
esac

tar xjf distfiles/$MPFR.tar.bz2
mv $MPFR $GCC/mpfr

tar xjf distfiles/$GMP.tar.bz2
mv $GMP $GCC/gmp

tar xzf distfiles/$MPC.tar.gz
mv $MPC $GCC/mpc

if [ -f diffs/$GCC.diff ]
then
	applypatches diffs/$GCC.diff
else
	echo "no diffs for $GCC"
	exit 1
fi

rm -rf gobjdir

mkdir gobjdir

cd gobjdir

$TOPDIR/$GCC/configure --prefix=$PREFIX --target=$TARGET \
	--enable-languages=c \
	--disable-nls --disable-shared --disable-tls \
	--with-sysroot --with-system-zlib \
	--with-gnu-as --with-gnu-ld --with-stabs \
	CFLAGS_FOR_TARGET="-O2 -fno-reorder-blocks -D_POSIX_SOURCE -D_MINIX"

$MAKE $PAR
$MAKE install

cd $TOPDIR

cd minix-1.6.25/usr/src/lib

OPATH=$PATH
export PATH=$PREFIX/bin:$PATH

$MAKE clean

for LIB in $($TARGET-gcc --print-multi-lib)
do
	LOCATION=$SYSROOT/usr/lib/$(echo $LIB | sed -e 's/;.*$//')
	XCFLAGS=$(echo $LIB | sed -e 's/^.*;//' -e 's/@/ -/g')
	case $GCC in
	gcc-[5678]*)
		XCFLAGS="$XCFLAGS -fno-builtin"
		;;
	*)
		;;
	esac
	$MAKE depend "XCFLAGS=$XCFLAGS" "SYSROOT=$SYSROOT"
	$MAKE "XCFLAGS=$XCFLAGS" "SYSROOT=$SYSROOT"
	mkdir -p $LOCATION
	cp crt0.o libc.a libd.a libm.a libcurses.a $LOCATION
	$MAKE clean "XCFLAGS=$XCFLAGS" "SYSROOT=$SYSROOT"
done

export PATH=$OPATH

cd $TOPDIR

cp -Rp $SYSROOT/usr/lib/* minix-1.6.25/usr/lib

rm -rf gobjdir

mkdir gobjdir

cd gobjdir


$TOPDIR/$GCC/configure --prefix=$PREFIX --target=$TARGET \
	--enable-languages=$LANGUAGES \
	--disable-nls --disable-shared --disable-tls \
	--with-sysroot --with-system-zlib \
	--with-gnu-as --with-gnu-ld --with-stabs \
	CFLAGS_FOR_TARGET="-O2 -fno-reorder-blocks -D_POSIX_SOURCE -D_MINIX" \
	CXXFLAGS_FOR_TARGET="-O2 -fno-reorder-blocks -D_POSIX_SOURCE -D_MINIX"

$MAKE $PAR
$MAKE install

cd $TOPDIR

# Move the linker script away from the sysroot to a more suitable location
mv $SYSROOT/usr/lib/ldscript \
	$PREFIX/lib/gcc/$TARGET/$(echo $GCC | sed -e 's/^gcc-//' -e 's/-.*$//')

# Leave environment settings in file devenv, to be either sourced or executed
# by the shell
echo "#!/bin/sh" > devenv
echo "export PATH=$PREFIX/bin:\$PATH" >> devenv
echo "export SYSROOT=$TOPDIR/minix-1.6.25" >> devenv
case $(uname -s) in
FreeBSD)
	if [ $LD_LIBRARY_PATH ]
	then
		echo "export LD_LIBRARY_PATH=$LD_LIBRARY_PATH" >> devenv
	fi
	;;
*)
	;;
esac

cat << 'END' >> devenv

case $(basename $(echo $0 | sed s/^-//)) in
devenv)
	SHELL=${SHELL-"/bin/sh"}
	exec $SHELL
	;;
*)
	;;
esac
END

# On FreeBSD, create symlink to man directory, so man(1) can find it
case $(uname -s) in
FreeBSD)
	ln -s $PREFIX/share/man $PREFIX/man
	;;
*)
	;;
esac

chmod +x devenv

echo
echo "$0: Done!"
